@@ -20,6 +20,9 @@
 #include "gconf-listeners.h"
 #include "gconf.h"
 
+#include <string.h>
+#include <unistd.h>
+
 /* #define DEBUG_LISTENERS 1 */
 #ifdef DEBUG_LISTENERS
 #include <stdio.h>
@@ -72,6 +75,7 @@ struct _LTableEntry {
   GList* listeners; /* Each listener listening *exactly* here. You probably 
                         want to notify all listeners *below* this node as well. 
                      */
+  gchar *full_name; /* fully-qualified name */
 };
 
 static LTable* ltable_new(void);
@@ -87,12 +91,20 @@ static void    ltable_notify(LTable* ltable,
                              gpointer user_data);
 
 static guint   ltable_next_cnxn(LTable* ltable);
+static void    ltable_foreach  (LTable* ltable,
+                                GConfListenersForeach callback,
+                                gpointer user_data);
+static gboolean ltable_get_data (LTable *ltable,
+                                 guint cnxn_id,
+                                 gpointer *listener_data_p,
+                                 const gchar  **location_p);
 
 #ifdef DEBUG_LISTENERS
 static void    ltable_spew(LTable* ltable);
 #endif
 
-static LTableEntry* ltable_entry_new(const gchar* name);
+static LTableEntry* ltable_entry_new(const gchar* name,
+                                     const gchar* full_name);
 static void         ltable_entry_destroy(LTableEntry* entry);
 
 static Listener* listener_new(guint cnxn_id, gpointer listener_data, GFreeFunc destroy_notify);
@@ -113,7 +125,7 @@ gconf_listeners_new     (void)
 }
 
 void
-gconf_listeners_destroy (GConfListeners* listeners)
+gconf_listeners_free (GConfListeners* listeners)
 {
   LTable* lt = (LTable*)listeners;
 
@@ -129,7 +141,9 @@ gconf_listeners_add     (GConfListeners* listeners,
   LTable* lt = (LTable*)listeners;
   Listener* l;
 
-  l = listener_new(ltable_next_cnxn(lt), listener_data, destroy_notify);
+  l = listener_new(ltable_next_cnxn(lt),
+                   listener_data,
+                   destroy_notify);
   
   ltable_insert(lt, listen_point, l);
 
@@ -171,8 +185,30 @@ gconf_listeners_count   (GConfListeners* listeners)
   return lt->active_listeners;
 }
 
+void
+gconf_listeners_foreach (GConfListeners* listeners,
+                         GConfListenersForeach callback,
+                         gpointer user_data)
+{
+  LTable* lt = (LTable*)listeners;
+  
+  ltable_foreach (lt, callback, user_data);
+}
+
+gboolean
+gconf_listeners_get_data (GConfListeners* listeners,
+                          guint cnxn_id,
+                          gpointer *listener_data_p,
+                          const gchar **location_p)
+{
+  LTable* lt = (LTable*)listeners;
+
+  return ltable_get_data (lt, cnxn_id, listener_data_p, location_p);
+}
+
 /*
- * LTable impl */
+ * LTable impl
+ */
 
 static Listener* 
 listener_new(guint cnxn_id, gpointer listener_data, GFreeFunc destroy_notify)
@@ -220,8 +256,19 @@ ltable_new(void)
 static guint
 ltable_next_cnxn(LTable* lt)
 {
+  static guchar start = 0;
   static guchar uniqueness = 0;
   guint uniqueness_shifted;
+
+  if (start == 0)
+    {
+      /* Don't start uniqueness at the same place every time */
+      start = getpid () % 256;
+      if (start == 0)
+        start = 1;
+
+      uniqueness = start;
+    }
   
   /* this overflows and starts over occasionally */
   ++uniqueness;
@@ -264,7 +311,7 @@ ltable_insert(LTable* lt, const gchar* where, Listener* l)
 
   if (lt->tree == NULL)
     {
-      lte = ltable_entry_new("/");
+      lte = ltable_entry_new("/", "/");
       
       lt->tree = g_node_new(lte);
 
@@ -313,7 +360,7 @@ ltable_insert(LTable* lt, const gchar* where, Listener* l)
 
       if (found == NULL)
         {
-          ne = ltable_entry_new(dirnames[i]);
+          ne = ltable_entry_new(dirnames[i], where);
               
           if (across != NULL) /* Across is at the one past */
             found = g_node_insert_data_before(cur, across, ne);
@@ -491,7 +538,11 @@ ltable_destroy(LTable* ltable)
 }
 
 static void
-notify_listener_list(GConfListeners* listeners, GList* list, const gchar* key, GConfListenersCallback callback, gpointer user_data)
+notify_listener_list(GConfListeners* listeners,
+                     GList* list,
+                     const gchar* key,
+                     GConfListenersCallback callback,
+                     gpointer user_data)
 {
   GList* tmp;
 
@@ -562,15 +613,116 @@ ltable_notify(LTable* lt, const gchar* key,
   g_strfreev(dirs);
 }
 
+struct NodeTraverseData
+{
+  GConfListenersForeach func;
+  gpointer user_data;
+};
+
+static gboolean
+node_traverse_func (GNode *node,
+                    gpointer user_data)
+{
+  struct NodeTraverseData *td = user_data;
+  LTableEntry *lte = node->data;
+  GList *tmp;
+
+  tmp = lte->listeners;
+  while (tmp != NULL)
+    {
+      Listener *l = tmp->data;
+      
+      (* td->func) (lte->full_name,
+                    l->cnxn,
+                    l->listener_data,
+                    td->user_data);
+      
+      tmp = g_list_next (tmp);
+    }
+
+  return FALSE; /* continue traversal */
+}
+
+static void
+ltable_foreach  (LTable* ltable,
+                 GConfListenersForeach callback,
+                 gpointer user_data)
+{
+  struct NodeTraverseData td;
+  td.func = callback;
+  td.user_data = user_data;
+
+  if (ltable->tree == NULL)
+    return;
+  
+  g_node_traverse (ltable->tree,
+                   G_IN_ORDER,
+                   G_TRAVERSE_ALL,
+                   -1,
+                   node_traverse_func,
+                   &td);
+}
+
+static gboolean
+ltable_get_data (LTable *lt,
+                 guint cnxn_id,
+                 gpointer *listener_data_p,
+                 const gchar   **location_p)
+{
+  LTableEntry* lte;
+  GList* tmp;
+  GNode* node;
+  guint index = CNXN_ID_INDEX(cnxn_id);
+  
+  g_return_val_if_fail(index < lt->listeners->len, FALSE);
+  if (index >= lt->listeners->len) /* robust even with checks off */
+    return FALSE;
+  
+  /* Lookup in the flat table */
+  node = g_ptr_array_index(lt->listeners, index);
+  
+  g_return_val_if_fail(node != NULL, FALSE);
+  if (node == NULL) /* a client is broken probably */
+    return FALSE;
+
+  g_assert(lt->tree != NULL);
+  
+  lte = node->data;
+  
+  tmp = lte->listeners;
+
+  g_return_val_if_fail(tmp != NULL, FALSE);
+
+  while (tmp != NULL)
+    {
+      Listener* l = tmp->data;
+
+      if (l->cnxn == cnxn_id)
+        {
+          if (listener_data_p)
+            *listener_data_p = l->listener_data;
+          if (location_p)
+            *location_p = lte->full_name;
+          return TRUE;
+        }
+
+      tmp = g_list_next(tmp);
+    }
+
+  return FALSE;
+}
+
 static LTableEntry* 
-ltable_entry_new(const gchar* name)
+ltable_entry_new(const gchar* name,
+                 const gchar *full_name)
 {
   LTableEntry* lte;
 
   lte = g_new0(LTableEntry, 1);
 
   lte->name = g_strdup(name);
-
+  lte->full_name = g_strdup (full_name);
+  
   return lte;
 }
 
@@ -579,6 +731,7 @@ ltable_entry_destroy(LTableEntry* lte)
 {
   g_return_if_fail(lte->listeners == NULL); /* should destroy all listeners first. */
   g_free(lte->name);
+  g_free(lte->full_name);
   g_free(lte);
 }
 
