@@ -428,7 +428,8 @@ gconf_sources_query_value (GConfSources* sources,
       
       source = tmp->data;
 
-      if (source_is_writable (source, key, NULL)) /* ignore errors */
+      if (value_is_writable &&
+          source_is_writable (source, key, NULL)) /* ignore errors */
         *value_is_writable = TRUE;
       
       val = gconf_source_query_value(source, key, locales,
@@ -852,8 +853,11 @@ gconf_sources_all_entries   (GConfSources* sources,
   gboolean first_pass = TRUE; /* as an optimization, don't bother
                                  doing hash lookups on first source
                               */
-  struct DefaultsLookupData dld = { sources, locales };
+  struct DefaultsLookupData dld = { NULL, NULL };
   
+  dld.sources = sources;
+  dld.locales = locales;
+
   /* Empty GConfSources, skip it */
   if (sources->sources == NULL)
     return NULL;
