@@ -26,20 +26,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-/* Quick hack so I can mark strings */
-
-#ifdef _ 
-#warning "_ already defined"
-#else
-#define _(x) x
-#endif
-
-#ifdef N_ 
-#warning "N_ already defined"
-#else
-#define N_(x) x
-#endif
-
 static void
 set_string(gchar** dest, const gchar* src)
 {
@@ -58,7 +44,7 @@ gconf_value_new(GConfValueType type)
 {
   GConfValue* value;
 
-  g_return_val_if_fail(GCONF_INTERNAL_VALUE_TYPE_VALID(type), NULL);
+  g_return_val_if_fail(GCONF_VALUE_TYPE_VALID(type), NULL);
   
   /* Probably want to use mem chunks here eventually. */
   value = g_new0(GConfValue, 1);
@@ -73,7 +59,7 @@ gconf_value_new(GConfValueType type)
 
 GConfValue* 
 gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
-                             GConfError** err)
+                             GError** err)
 {
   GConfValue* value;
 
@@ -96,7 +82,7 @@ gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
                                       _("Didn't understand `%s' (expected integer)"),
                                       value_str);
             
-            gconf_value_destroy(value);
+            gconf_value_free(value);
             value = NULL;
           }
         else if (errno == ERANGE)
@@ -105,7 +91,7 @@ gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
               *err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
                                       _("Integer `%s' is too large or small"),
                                       value_str);
-            gconf_value_destroy(value);
+            gconf_value_free(value);
             value = NULL;
           }
         else
@@ -127,7 +113,7 @@ gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
                                       _("Didn't understand `%s' (expected real number)"),
                                      value_str);
             
-            gconf_value_destroy(value);
+            gconf_value_free(value);
             value = NULL;
           }
       }
@@ -160,7 +146,7 @@ gconf_value_new_from_string(GConfValueType type, const gchar* value_str,
                                    _("Didn't understand `%s' (expected true or false)"),
                                    value_str);
           
-          gconf_value_destroy(value);
+          gconf_value_free(value);
           value = NULL;
           break;
         }
@@ -210,7 +196,7 @@ escape_string(const char *str, const char *escaped_chars)
 GConfValue*
 gconf_value_new_list_from_string(GConfValueType list_type,
                                   const gchar* str,
-				  GConfError** err)
+				  GError** err)
 {
   int i, len;
   gboolean escaped, pending_chars;
@@ -268,7 +254,7 @@ gconf_value_new_list_from_string(GConfValueType list_type,
 	  if (err && *err != NULL)
 	    {
 	      /* Free values so far */
-	      g_slist_foreach(list, (GFunc)gconf_value_destroy, NULL);
+	      g_slist_foreach(list, (GFunc)gconf_value_free, NULL);
 	      g_slist_free(list);
 
 	      g_string_free(string, TRUE);
@@ -282,7 +268,7 @@ gconf_value_new_list_from_string(GConfValueType list_type,
 	      i != len-1)
 	    {
 	      /* Free values so far */
-	      g_slist_foreach(list, (GFunc)gconf_value_destroy, NULL);
+	      g_slist_foreach(list, (GFunc)gconf_value_free, NULL);
 	      g_slist_free(list);
 
 	      g_string_free(string, TRUE);
@@ -313,7 +299,7 @@ gconf_value_new_list_from_string(GConfValueType list_type,
   if (pending_chars)
     {
       /* Free values so far */
-      g_slist_foreach(list, (GFunc)gconf_value_destroy, NULL);
+      g_slist_foreach(list, (GFunc)gconf_value_free, NULL);
       g_slist_free(list);
 
       g_string_free(string, TRUE);
@@ -340,7 +326,7 @@ GConfValue*
 gconf_value_new_pair_from_string(GConfValueType car_type,
                                   GConfValueType cdr_type,
                                   const gchar* str,
-				  GConfError** err)
+				  GError** err)
 {
   int i, len;
   int elem;
@@ -394,9 +380,9 @@ gconf_value_new_pair_from_string(GConfValueType car_type,
 	    {
 	      /* Free values so far */
 	      if (car)
-	        gconf_value_destroy(car);
+	        gconf_value_free(car);
 	      if (cdr)
-	        gconf_value_destroy(cdr);
+	        gconf_value_free(cdr);
 
 	      g_string_free(string, TRUE);
 
@@ -418,9 +404,9 @@ gconf_value_new_pair_from_string(GConfValueType car_type,
 	    {
 	      /* Free values so far */
 	      if (car)
-	        gconf_value_destroy(car);
+	        gconf_value_free(car);
 	      if (cdr)
-	        gconf_value_destroy(cdr);
+	        gconf_value_free(cdr);
 
 	      g_string_free(string, TRUE);
 
@@ -434,9 +420,9 @@ gconf_value_new_pair_from_string(GConfValueType car_type,
 	    {
 	      /* Free values so far */
 	      if (car)
-	        gconf_value_destroy(car);
+	        gconf_value_free(car);
 	      if (cdr)
-	        gconf_value_destroy(cdr);
+	        gconf_value_free(cdr);
 
 	      g_string_free(string, TRUE);
 
@@ -467,9 +453,9 @@ gconf_value_new_pair_from_string(GConfValueType car_type,
     {
       /* Free values so far */
       if (car)
-	gconf_value_destroy(car);
+	gconf_value_free(car);
       if (cdr)
-	gconf_value_destroy(cdr);
+	gconf_value_free(cdr);
 
       if (err)
 	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
@@ -482,9 +468,9 @@ gconf_value_new_pair_from_string(GConfValueType car_type,
     {
       /* Free values so far */
       if (car)
-	gconf_value_destroy(car);
+	gconf_value_free(car);
       if (cdr)
-	gconf_value_destroy(cdr);
+	gconf_value_free(cdr);
 
       if (err)
 	*err = gconf_error_new(GCONF_ERROR_PARSE_ERROR,
@@ -512,22 +498,22 @@ gconf_value_to_string(GConfValue* value)
   switch (value->type)
     {
     case GCONF_VALUE_INT:
-      retval = g_strdup_printf("%d", gconf_value_int(value));
+      retval = g_strdup_printf("%d", gconf_value_get_int(value));
       break;
     case GCONF_VALUE_FLOAT:
-      retval = gconf_double_to_string(gconf_value_float(value));
+      retval = gconf_double_to_string(gconf_value_get_float(value));
       break;
     case GCONF_VALUE_STRING:
-      retval = g_strdup(gconf_value_string(value));
+      retval = g_strdup(gconf_value_get_string(value));
       break;
     case GCONF_VALUE_BOOL:
-      retval = gconf_value_bool(value) ? g_strdup("true") : g_strdup("false");
+      retval = gconf_value_get_bool(value) ? g_strdup("true") : g_strdup("false");
       break;
     case GCONF_VALUE_LIST:
       {
         GSList* list;
 
-        list = gconf_value_list(value);
+        list = gconf_value_get_list(value);
 
         if (list == NULL)
           retval = g_strdup("[]");
@@ -600,10 +586,10 @@ gconf_value_to_string(GConfValue* value)
         gchar* car;
         gchar* cdr;
 
-        tmp = gconf_value_to_string(gconf_value_car(value));
+        tmp = gconf_value_to_string(gconf_value_get_car(value));
 	car = escape_string(tmp, ",)");
 	g_free(tmp);
-        tmp = gconf_value_to_string(gconf_value_cdr(value));
+        tmp = gconf_value_to_string(gconf_value_get_cdr(value));
 	cdr = escape_string(tmp, ",)");
 	g_free(tmp);
         retval = g_strdup_printf("(%s,%s)", car, cdr);
@@ -623,11 +609,11 @@ gconf_value_to_string(GConfValue* value)
         const gchar* car_type;
         const gchar* cdr_type;
         
-        locale = gconf_schema_locale(gconf_value_schema(value));
-        type = gconf_value_type_to_string(gconf_schema_type(gconf_value_schema(value)));
-        list_type = gconf_value_type_to_string(gconf_schema_list_type(gconf_value_schema(value)));
-        car_type = gconf_value_type_to_string(gconf_schema_car_type(gconf_value_schema(value)));
-        cdr_type = gconf_value_type_to_string(gconf_schema_cdr_type(gconf_value_schema(value)));
+        locale = gconf_schema_get_locale(gconf_value_get_schema(value));
+        type = gconf_value_type_to_string(gconf_schema_get_type(gconf_value_get_schema(value)));
+        list_type = gconf_value_type_to_string(gconf_schema_get_list_type(gconf_value_get_schema(value)));
+        car_type = gconf_value_type_to_string(gconf_schema_get_car_type(gconf_value_get_schema(value)));
+        cdr_type = gconf_value_type_to_string(gconf_schema_get_cdr_type(gconf_value_get_schema(value)));
         
         retval = g_strdup_printf("Schema (type: `%s' list_type: '%s' "
 				 "car_type: '%s' cdr_type: '%s' locale: `%s')",
@@ -731,7 +717,7 @@ gconf_value_free_list(GConfValue* value)
 
   while (tmp != NULL)
     {
-      gconf_value_destroy(tmp->data);
+      gconf_value_free(tmp->data);
       
       tmp = g_slist_next(tmp);
     }
@@ -741,7 +727,7 @@ gconf_value_free_list(GConfValue* value)
 }
 
 void 
-gconf_value_destroy(GConfValue* value)
+gconf_value_free(GConfValue* value)
 {
   g_return_if_fail(value != NULL);
 
@@ -753,17 +739,17 @@ gconf_value_destroy(GConfValue* value)
       break;
     case GCONF_VALUE_SCHEMA:
       if (value->d.schema_data != NULL)
-        gconf_schema_destroy(value->d.schema_data);
+        gconf_schema_free(value->d.schema_data);
       break;
     case GCONF_VALUE_LIST:
       gconf_value_free_list(value);
       break;
     case GCONF_VALUE_PAIR:
       if (value->d.pair_data.car != NULL)
-        gconf_value_destroy(value->d.pair_data.car);
+        gconf_value_free(value->d.pair_data.car);
 
       if (value->d.pair_data.cdr != NULL)
-        gconf_value_destroy(value->d.pair_data.cdr);
+        gconf_value_free(value->d.pair_data.cdr);
       break;
     default:
       break;
@@ -815,7 +801,7 @@ gconf_value_set_schema(GConfValue* value, GConfSchema* sc)
   g_return_if_fail(value->type == GCONF_VALUE_SCHEMA);
   
   if (value->d.schema_data != NULL)
-    gconf_schema_destroy(value->d.schema_data);
+    gconf_schema_free(value->d.schema_data);
 
   value->d.schema_data = gconf_schema_copy(sc);
 }
@@ -828,7 +814,7 @@ gconf_value_set_schema_nocopy(GConfValue* value, GConfSchema* sc)
   g_return_if_fail(sc != NULL);
   
   if (value->d.schema_data != NULL)
-    gconf_schema_destroy(value->d.schema_data);
+    gconf_schema_free(value->d.schema_data);
 
   value->d.schema_data = sc;
 }
@@ -849,7 +835,7 @@ gconf_value_set_car_nocopy(GConfValue* value, GConfValue* car)
   g_return_if_fail(car != NULL);
   
   if (value->d.pair_data.car != NULL)
-    gconf_value_destroy(value->d.pair_data.car);
+    gconf_value_free(value->d.pair_data.car);
 
   value->d.pair_data.car = car;
 }
@@ -870,7 +856,7 @@ gconf_value_set_cdr_nocopy(GConfValue* value, GConfValue* cdr)
   g_return_if_fail(cdr != NULL);
   
   if (value->d.pair_data.cdr != NULL)
-    gconf_value_destroy(value->d.pair_data.cdr);
+    gconf_value_free(value->d.pair_data.cdr);
 
   value->d.pair_data.cdr = cdr;
 }
@@ -939,7 +925,7 @@ gconf_meta_info_new(void)
 }
 
 void
-gconf_meta_info_destroy(GConfMetaInfo* gcmi)
+gconf_meta_info_free(GConfMetaInfo* gcmi)
 {
   g_free(gcmi);
 }
@@ -969,6 +955,15 @@ gconf_meta_info_set_mod_time(GConfMetaInfo* gcmi,
  * GConfEntry
  */
 
+GConfEntry*
+gconf_entry_new (const gchar *key,
+                 GConfValue  *val)
+{
+  return gconf_entry_new_nocopy (g_strdup (key),
+                                 val ? gconf_value_copy (val) : NULL);
+
+}
+
 GConfEntry* 
 gconf_entry_new_nocopy(gchar* key, GConfValue* val)
 {
@@ -980,16 +975,17 @@ gconf_entry_new_nocopy(gchar* key, GConfValue* val)
   pair->value = val;
   pair->schema_name = NULL;
   pair->is_default = FALSE;
+  pair->is_writable = TRUE;
   
   return pair;
 }
 
 void
-gconf_entry_destroy(GConfEntry* pair)
+gconf_entry_free(GConfEntry* pair)
 {
   g_free(pair->key);
   if (pair->value)
-    gconf_value_destroy(pair->value);
+    gconf_value_free(pair->value);
   g_free(pair);
 }
 
@@ -1001,12 +997,20 @@ gconf_entry_steal_value (GConfEntry* entry)
   return val;
 }
 
+void
+gconf_entry_set_value (GConfEntry  *entry,
+                       GConfValue  *val)
+{
+  gconf_entry_set_value_nocopy (entry,
+                                val ? gconf_value_copy (val) : NULL);
+}
+
 void
 gconf_entry_set_value_nocopy(GConfEntry* entry,
                              GConfValue* val)
 {
   if (entry->value)
-    gconf_value_destroy(entry->value);
+    gconf_value_free(entry->value);
 
   entry->value = val;
 }
