@@ -1250,7 +1250,7 @@ gconf_engine_all_entries(GConfEngine* conf, const gchar* dir, GError** err)
   if (keys->_length != values->_length)
     {
       g_warning("Received unmatched key/value sequences in %s",
-                __FUNCTION__);
+                G_GNUC_FUNCTION);
       return NULL;
     }
 
@@ -1892,10 +1892,10 @@ static void
 update_listener (PortableServer_Servant _servant,
                  ConfigDatabase             db,
                  const CORBA_char          *address,
-                 const CORBA_unsigned_long old_cnxn_id,
-                 const CORBA_char * key,
-                 const CORBA_unsigned_long new_cnxn_id,
-                 CORBA_Environment * ev_ignored)
+                 const CORBA_unsigned_long  old_cnxn_id,
+                 const CORBA_char          *key,
+                 const CORBA_unsigned_long  new_cnxn_id,
+                 CORBA_Environment         *ev_ignored)
 {
   GConfCnxn* cnxn;
   GConfEngine* conf;
@@ -1922,7 +1922,8 @@ update_listener (PortableServer_Servant _servant,
   if (conf == NULL)
     {
 #ifdef GCONF_ENABLE_DEBUG
-      g_warning("Client received listener update for unknown database");
+      g_warning("Client received listener update for unknown database "
+                "(this is not a big deal, this warning only appears if GConf is compiled with debugging)");
 #endif
       return;
     }
@@ -1932,7 +1933,8 @@ update_listener (PortableServer_Servant _servant,
   if (cnxn == NULL)
     {
 #ifdef GCONF_ENABLE_DEBUG
-      g_warning("Client received listener update for unknown listener ID %u",
+      g_warning("Client received listener update for unknown listener ID %u "
+                "(this is not a big deal, this warning only appears if GConf is compiled with debugging)",
                 (guint)old_cnxn_id);
 #endif
       return;
@@ -1947,14 +1949,18 @@ invalidate_cached_values (PortableServer_Servant     _servant,
                           const ConfigListener_KeyList *keys,
                           CORBA_Environment         *ev)
 {
+#if 0
   g_warning ("FIXME process %d received request to invalidate some cached GConf values from the server, but right now we don't know how to do that (not implemented).", (int) getpid());
+#endif
 }
 
 static void
 drop_all_caches (PortableServer_Servant     _servant,
                  CORBA_Environment         *ev)
 {
+#if 0
   g_warning ("FIXME process %d received request to invalidate all cached GConf values from the server, but right now we don't know how to do that (not implemented).", (int) getpid());
+#endif
 }
 
 static ConfigListener 
@@ -2242,8 +2248,9 @@ ctable_new(void)
 
   ct = g_new(CnxnTable, 1);
 
-  ct->server_ids = g_hash_table_new(corba_unsigned_long_hash, corba_unsigned_long_equal);  
-  ct->client_ids = g_hash_table_new(g_int_hash, g_int_equal);
+  ct->server_ids = g_hash_table_new (corba_unsigned_long_hash,
+                                     corba_unsigned_long_equal);  
+  ct->client_ids = g_hash_table_new (g_int_hash, g_int_equal);
   
   return ct;
 }
@@ -2251,23 +2258,23 @@ ctable_new(void)
 static void
 ctable_destroy(CnxnTable* ct)
 {
-  g_hash_table_destroy(ct->server_ids);
-  g_hash_table_destroy(ct->client_ids);
+  g_hash_table_destroy (ct->server_ids);
+  g_hash_table_destroy (ct->client_ids);
   g_free(ct);
 }
 
 static void       
 ctable_insert(CnxnTable* ct, GConfCnxn* cnxn)
 {
-  g_hash_table_insert(ct->server_ids, &cnxn->server_id, cnxn);
-  g_hash_table_insert(ct->client_ids, &cnxn->client_id, cnxn);
+  g_hash_table_insert (ct->server_ids, &cnxn->server_id, cnxn);
+  g_hash_table_insert (ct->client_ids, &cnxn->client_id, cnxn);
 }
 
 static void       
 ctable_remove(CnxnTable* ct, GConfCnxn* cnxn)
 {
-  g_hash_table_remove(ct->server_ids, &cnxn->server_id);
-  g_hash_table_remove(ct->client_ids, &cnxn->client_id);
+  g_hash_table_remove (ct->server_ids, &cnxn->server_id);
+  g_hash_table_remove (ct->client_ids, &cnxn->client_id);
 }
 
 static void       
@@ -2275,11 +2282,11 @@ ctable_remove_by_client_id(CnxnTable* ct, guint client_id)
 {
   GConfCnxn* cnxn;
 
-  cnxn = ctable_lookup_by_client_id(ct, client_id);
+  cnxn = ctable_lookup_by_client_id (ct, client_id);
 
-  g_return_if_fail(cnxn != NULL);
+  g_return_if_fail (cnxn != NULL);
 
-  ctable_remove(ct, cnxn);
+  ctable_remove (ct, cnxn);
 }
 
 struct RemoveData {
@@ -2322,11 +2329,15 @@ ctable_remove_by_conf(CnxnTable* ct, GConfEngine* conf)
   rd.conf = conf;
   rd.save_removed = TRUE;
   
-  client_ids_removed = g_hash_table_foreach_remove(ct->server_ids, remove_by_conf, &rd);
+  client_ids_removed = g_hash_table_foreach_remove (ct->server_ids,
+                                                    remove_by_conf,
+                                                    &rd);
 
   rd.save_removed = FALSE;
 
-  server_ids_removed = g_hash_table_foreach_remove(ct->client_ids, remove_by_conf, &rd);
+  server_ids_removed = g_hash_table_foreach_remove(ct->client_ids,
+                                                   remove_by_conf,
+                                                   &rd);
 
   g_assert(client_ids_removed == server_ids_removed);
   g_assert(client_ids_removed == g_slist_length(rd.removed));
@@ -2343,7 +2354,7 @@ ctable_lookup_by_client_id(CnxnTable* ct, guint client_id)
 static GConfCnxn* 
 ctable_lookup_by_server_id(CnxnTable* ct, CORBA_unsigned_long server_id)
 {
-  return g_hash_table_lookup(ct->server_ids, &server_id);
+  return g_hash_table_lookup (ct->server_ids, &server_id);
 }
 
 static void
@@ -2354,9 +2365,10 @@ ctable_reinstall (CnxnTable* ct,
 {
   g_return_if_fail (cnxn->server_id == old_server_id);
 
+  g_hash_table_remove (ct->server_ids, &old_server_id);
+  
   cnxn->server_id = new_server_id;
 
-  g_hash_table_remove (ct->server_ids, &old_server_id);
   g_hash_table_insert (ct->server_ids, &cnxn->server_id, cnxn);
 }
 
@@ -2534,7 +2546,7 @@ gconf_engine_get_string(GConfEngine* conf, const gchar* key,
 
 gboolean     
 gconf_engine_get_bool  (GConfEngine* conf, const gchar* key,
-                 GError** err)
+                        GError** err)
 {
   GConfValue* val;
   static const gboolean deflt = FALSE;
@@ -2884,6 +2896,9 @@ corba_errno_to_gconf_errno(ConfigErrorType corba_err)
     case ConfigNoWritableDatabase:
       return GCONF_ERROR_NO_WRITABLE_DATABASE;
       break;
+    case ConfigInShutdown:
+      return GCONF_ERROR_IN_SHUTDOWN;
+      break;
     default:
       g_assert_not_reached();
       return GCONF_ERROR_SUCCESS; /* warnings */
@@ -2899,6 +2914,17 @@ gconf_server_broken(CORBA_Environment* ev)
     case CORBA_SYSTEM_EXCEPTION:
       return TRUE;
       break;
+
+    case CORBA_USER_EXCEPTION:
+      {
+        ConfigException* ce;
+
+        ce = CORBA_exception_value(ev);
+
+        return ce->err_no == ConfigInShutdown;
+      }
+      break;
+      
     default:
       return FALSE;
       break;
